#include <u.h>
#include <libc.h>

/*
Decomp decomposes composed flags and options in the arguments,
so that make easy to parse flags and options in rc script.
usage: decomp -s sep
the sep denotes output separater used only for debugging. the default is '\t'.

typical usage is in rc script:

nl='
'	# NL
*=`$nl{decomp -s $nl flg:opt $*}
if(~ $status ?*) { do something }
while(~ $1 -?*)
  switch($1){
  case a
	...
  case b
	...
  }
if(~ $1 -)
	shift

2021/12/14 Kenar

Other example:
decomp rw:a -rwabc - -def
results:
	-r	-w	-a	bc	-	-def
*/

char *flg, *opt;
int cont = 0;
int nflag = 0;

char *sep = "\t";
char *fmt1="\n# error in last option %s. need the value\n";
char *fmt2 = "\n# error in %c\t(-%s)\n";

void
usage(void)
{
	fprint(2, "usage: decomp [-n] [-s sep ] flg:opt [arg ...]\n");
	exits("usage");
}

void
error(char *fmt, ...)
{
	va_list args;
	va_start(args, fmt);
	vfprint(2,fmt,args);
	va_end(args);
	exits("syntax");
}

char *
split(char *s, int c)
{
	char *p;
	p = strchr(s,c);
	if(p == nil)
		return nil;
	*p = 0;
	p++;
	return p;
}

int
decomp1(char *s)
{
	char *s0, b;
	if(cont){
		print("%s%s", sep, s);
		cont = 0;
		return 1;
	}
	if(strcmp(s,"-") == 0 || strcmp(s,"--") == 0){
		print("%s-",sep);
		return 0;
	}
	if(*s != '-'){
		print("%s%s", sep, s);
		return 0;
	}
	s++;
	cont = 0;
	s0 = strdup(s);
	for(;;){
		b = *s++;
		/* NB: need b==0 */
		if(b == 0 || strchr(flg,b) == nil)
			break;
		print("%s-%c", sep, b);
	}
	if(!b){
		free(s0);
		return 1;
	}
	/* then next char should be an option */
	if(strchr(opt,b) == nil){ /* not an option */
		error(fmt2, b, s0);
		free(s0);
		return 0;
	}
	free(s0);
	print("%s-%c", sep, b);
	if(*s == 0){
		cont = 1;
		return 1;
	}
	print("%s%s", sep, s);
	return 1;
}

void
main(int argc, char *argv[])
{
	int i,rv;
	ARGBEGIN{
	case 's': /* output separater */
		sep = EARGF(usage());
		break;
	case 'n': /* trailing "\n" */
		nflag = 1;
		break;
	default:
		usage();
	}ARGEND

	if(argc == 0)
		usage();

	flg = strdup(argv[0]);
	opt = split(flg,':');
	if(opt == nil)
		usage();
	rv = 1;
	for(i=1; rv && i < argc; i++)
		rv = decomp1(argv[i]);
	for(; i < argc; i++)
		print("%s%s", sep, argv[i]);
	if(cont == 1)
		error(fmt1,argv[argc - 1]);
	if(nflag)
		print("\n");
	exits(nil);
}
